-- $Id: generic_beam.lua 4252 2021-07-31 23:22:57Z dsb $
-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009 - 2019 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

-- Modified from T4's 'mana_beam' particle to allow particle color to be
-- specified.  Particle parameters:
--
--   - dx, dy:		x/y distance (in grid coordinates) from source
--   			location to target location.
--   - rr, gg, bb:	Red/green/blue levels (range 0-255) to use for the
--   			particle.

-- Make the ray
local ray = {}
local tiles = math.ceil(math.sqrt(dx*dx+dy*dy))
local dx = dx * engine.Map.tile_w
local dy = dy * engine.Map.tile_h
local breakdir = math.rad(rng.range(-8, 8))
ray.dir = math.atan2(dy, dx)
ray.size = math.sqrt(dx*dx+dy*dy)

-- Populate the beam based on the forks
return { generator = function()
	local a = ray.dir
	local rad = rng.range(-3,3)
	local ra = math.rad(rad)
	local r = rng.range(1, ray.size)

	return {
		life = 14,
		size = rng.range(4, 6), sizev = -0.1, sizea = 0,

		x = r * math.cos(a) + 2 * math.cos(ra), xv = 0, xa = 0,
		y = r * math.sin(a) + 2 * math.sin(ra), yv = 0, ya = 0,
		dir = rng.percent(50) and ray.dir + math.rad(rng.range(50, 130)) or ray.dir - math.rad(rng.range(50, 130)), dirv = 0, dira = 0,
		vel = rng.percent(30) and 1 or 0, velv = -0.1, vela = 0.01,

		r = rr/255,  rv = 0, ra = 0,
		g = gg/255,  gv = 0, ga = 0,
		b = bb/255,  bv = 0, ba = 0,
		a = rng.range(80, 220)/255,   av = 0, aa = 0,
	}
end, },
function(self)
	self.nb = (self.nb or 0) + 1
	if self.nb < 6 then
		self.ps:emit(30*tiles)
	end
end,
14*30*tiles,
"particle_torus"
