-- $Id: bree-nazgul.lua 3883 2020-04-20 00:24:34Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

add_hooks = function()
  class:bindHook('CharacterSheet:dumpLine', function(self, data)
    local q = game.player:hasQuest('bree-nazgul')
    if q and q:isEnded() then
      table.insert(data.lines, [[You saved Bree from a dreadful Nazgûl.]])
    end
  end)
end

id = 'bree-nazgul'
name = 'The Dark Horseman'
danger_level = 40	-- cf. src/tables.c line 3796
desc = function(self, who)
  if self:isEnded() then
    return [[You have slain Uvatha the Horseman, who was lurking in the area of Bree.  Mayor Will Whitfoot thanks you and gave you some Athelas to help patch up the likely effects of fighting a Nazgûl.]]
  elseif self:isCompleted() then
    return [[You have slain Uvatha the Horseman, who was lurking in the area of Bree.  You should go back to the Mayor's office and tell the mayor.]]
  else
    return [[A dark-cloaked horseman has been spotted several times in Bree.  He carries an aura of fear with him and people seem to get sick wherever he gose.  You must slay him.]]
  end
end

on_grant = function(self, who)
  self:place_uvatha()
end

place_uvatha = function(self)
  if self:isEnded() or self:isCompleted() then return end

  local gen = game.zone:getGenerator('actor', game.level, game.level.data.generator.actor)
  local m = gen:generateOne {
    name = 'UVATHA_THE_HORSEMAN',
    force_unique = true,
    -- Not in LOS of the player.
    ok_location = function(x, y) return not game.player:hasLOS(x, y) end,
    ret = true,
  }
  if m then
    m.quest_mon = true
    m.quest_on_die = 'bree-nazgul'
  end
end

on_quest_mon_die = function(self, who, src)
  -- Killed Uvatha.
  game.player:setQuestStatus('bree-nazgul', engine.Quest.COMPLETED)
end
