-- $Id: nirnaeth-arnoediad.lua 3936 2020-07-10 00:43:30Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

id = 'nirnaeth-arnoediad'
name = 'Nirnaeth Arnoediad'
damger_level = 37	-- cf. src/tables.c line 3970

orig_count = 0

desc = function(self, who)
  local all = self:isCompleted('killed-all')
  local most = self:isCompleted('killed-most')
  local killed, money = '', ''
  if all or most then money = 'gave you some gold and ' end
  if all then
    killed = [[, slaughtering all of Morgoth's forces in the process]]
  elseif most then
    killed = [[, killing a large majority Morgoth's forces in the process]]
  end

  if self:isEnded() then
    return ([[You cleared a path through Morgoth's forces for Maedhros' army to retreat to Gondolin%s.  As a reward, Turgon %sgranted you access to the royal jewelry shop.]]):format(killed, money)
  elseif self:isCompleted() then
    return ([[You cleared a path through Morgoth's forces for Maedhros' army to retreat to Gondolin%s.  You should return to the Tower of the King and tell Turgon.]]):format(killed)
  elseif all then
    return [[Maedhros' army was routed at Thangorodrim and is trying to retreat to Gondolin, but Morgoth's forces are in the way.  Well, they were, until you slaughtered all of them...  Now get to the exit and lead Maedros' people to safety.]]
  elseif most then
    return [[Maedhros' army was routed at Thangorodrim and is trying to retreat to Gondolin, but Morgoth's forces are in the way.  You have killed a large majority of Morgoth's forces; now get to the exit and lead Maedros' people to safety.]]
  else
    return [[Maedhros' army was routed at Thangorodrim and is trying to retreat to Gondolin, but Morgoth's forces are in the way.  You must find a path through those forces and lead Maedhros' people to safety.  You don't need to kill Morgoth's forces (though you certainly can if you want to); just get to the exit.]]
  end
end

on_status_change = function(self, who, status, sub)
  if status == self.DONE then
    local all = self:isCompleted('killed-all')
    local most = self:isCompleted('killed-most')
    if all or most then who:adjGold(200000) end
    local tag = all and 'NIRNAETH_SLAUGHTER' or most and 'NIRNAETH_KILL' or 'NIRNAETH'
    world:gainAchievement(tag, who)
  end
end

on_quest_mon_die = function(self, who, src)
  -- Only interested in kills in the quest zone.
  if game.zone.short_name ~= 'quest-nirnaeth-arnoediad' then return end
  -- Count how many hostiles are left.
  local n = 0
  for _, e in pairs(game.level.entities) do
    if e.reactionToward and e:reactionToward(game.player) < 0 and not e.dead then
      n = n + 1
    end
  end
  if n == 0 and not self:isCompleted('killed-all') then
    game.player:setQuestStatus(self.id, self.COMPLETED, 'killed-all')
  elseif n*3 <= self.orig_count and not self:isCompleted('killed-most') then
    game.player:setQuestStatus(self.id, self.COMPLETED, 'killed-most')
  end
end

callbacks = {
  on_leave_level = function(who, q, zone, level, lev, params)
    if game.zone.short_name == 'quest-nirnaeth-arnoediad' then
      -- Leaving the quest zone; completion.
      game.logPlayer(who, '#YELLOW#You found a way out!#LAST#')
      who:setQuestStatus(q.id, q.COMPLETED)
    end
  end,
}
