-- $Id: greater_vault.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2016-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Roughly mimics build_type7() in src/generate.c, to build a greater vault.
newRoom {
  id = 'greater_vault',
  min_depth = 5,		-- cf. roomdep[] in src/generate.c
  wgt_very_unusual = 15,	-- cf. src/generate.c lines 6928-6939
  build = function(self, gen, gx, gy)
    local vdef = rng.table(gen.vault_types.greater)
    if not vdef then
      print('[MAPGEN] ??? no greater vault definitions?')
      return false
    end

    local ctr = gen:roomAlloc(vdef.width, vdef.height, false, gx, gy)
    if not ctr then
      print(("[MAPGEN] cannot fit greater vault '%s' @%d,%d"):format(vdef.id, gx, gy))
      return false
    end

    gen:report_precog('Greater vault')
    print(("[MAPGEN] Generating greater vault '%s' centered @%d,%d"):format(vdef.id, ctr.x, ctr.y))

    -- Increase level rating.
    gen.level.rating = (gen.level.rating or 0) + vdef.rating
    -- Also, depending on depth, possibly flag the level as "special".
    local dl40 = gen.dun_lev - 40
    if gen.dun_lev <= 40 or rng.range(1, dl40*dl40 + 50) < 400 then
      gen.level.special_feeling = true
    end

    -- Slight Hack(TM):  Keep a count of vaults we've created, to use to
    -- "unique-ify" jumpgate info for the vault
    gen.n_vaults = (gen.n_vaults or 0) + 1
    --
    -- TODO "Clean the between gates arrays" [cf. build_vault() in
    -- src/generate.c].  (This may not actually be necessary, given the
    -- above.)

    -- Build the vault.  This is loosely based on T4's 'lesser_vault' room.
    local Map = require 'mod.class.Map'
    local Static = require 'mod.class.generator.map.Static'
    local gen_data = table.clone(gen.data)
    gen_data.map = 'vaults/greater/'..vdef.map
    -- Make this info available to the room builder.
    gen_data.n_vaults = gen.n_vaults

    -- Questionable Hack(TM):  Create a new map to load the vault into, and
    -- wedge it in everywhere the generator will expect the level map to be.
    local vault_map = Map.new(vdef.width, vdef.height)
    local orig_gen_level_map = gen.level.map
    gen.level.map = vault_map
    local orig_game_level = game.level
    game.level = gen.level

    -- Dodgy Hack(TM):  Pass our provided map generator through to
    -- spell.wall_grid() for use in setting up secret doors in vaults.
    spell.map_generator = gen

    -- Create a new Static map generator and load our vault map.
    local room_gen = Static.new(gen.zone, vault_map, gen.level, gen_data)
    room_gen.chain_resolve = function(res)
      -- Slight Hack(TM):  Resolve 'outer_wall'/'inner_wall' for the Static
      -- generator.
      return gen:resolve(res, nil, false, gen.dun_lev)
    end
    local _, _, _, _, vault_spots = room_gen:generate(gen.cur_lev, gen.cur_lev)

    -- Dodgy Hack(TM), redux:  Clean up behind ourself.
    spell.map_generator = nil

    -- Questionable Hack(TM), redux:  Put back the actual level map.
    game.level = orig_game_level
    gen.level.map = orig_gen_level_map

    -- Overlay the vault map on top of the main map.
    local x1 = ctr.x - math.floor(vdef.width/2)
    local y1 = ctr.y - math.floor(vdef.height/2)
    print(("[MAPGEN]   placing vault @%d,%d"):format(x1, y1))
    gen.level.map:overlay(vault_map, x1, y1)

    -- I don't think any of the vault maps define spots, but just in case,
    -- fold them back into the level generator's spots list.
    for _, spot in ipairs(vault_spots) do
      spot.x = spot.x + x1
      spot.y = spot.y + y1
    end
    table.append(gen.spots, vault_spots)
    return true
  end,
}
