-- $Id: mimicry-cloak.lua 3132 2017-10-05 23:39:50Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Entity = require 'engine.Entity'

newTalentType {
  name = 'Mimicry cloak activation',
  type = 'player/mimicry-cloak',
  description = [[Mimicry powers conferred by wearing or using a Mimicry Cloak]],
}

-- This is done in ObjectData:newMimicryForm() now.
--[[
local function define_cloak_talent(name)
  local talent_name = name .. ' Form'
  local short_name = 'CLOAK_' .. name:upper():gsub("[ ']", '_')
  newTalent {
    name = talent_name,
    short_name = short_name,
    display_entity = Entity.new { image='talents/'..name:lower()..'_form.png' },
    type = 'player/mimicry-cloak',
    mode = 'sustained',
    is_spell = true,
    skill = 'MIMICRY',
    skill_level = 1,
    mana = 2,
    base_fail = 0,
    stat = 'DEX',
    points = 0,
    mimic_form = name,
    info = function(who, t)
      local ObjectData = require 'mod.class.interface.ObjectData'
      local mf = ObjectData.mimicry_forms[t.mimic_form]
      local lev = who:getSkillLevel('MIMICRY')
      local a = mf.name:gmatch('^[AEIOUaeiou]') and 'an' or 'a'
      local desc = tstring {
	('Transform into %s %s.'):format(a, mf.name),
	true, true, 'Effects at your current Mimicry skill level:', true
      }
      desc:merge(mf.info(who, lev))
      return desc:toString()
    end,
    sustain_describe = function(who, t, p)
      local ObjectData = require 'mod.class.interface.ObjectData'
      local mf = ObjectData.mimicry_forms[t.mimic_form]
      local lev = who:getSkillLevel('MIMICRY')
      local a = mf.name:gmatch('^[AEIOUaeiou]') and 'an' or 'a'
      local desc = tstring {
	('You are transformed into %s %s, with the following effects:'):format(a, mf.name),
	true
      }
      desc:merge(mf.info(who, lev))
      return desc
    end,
    activate = function(who, def)
      game.logPlayer(who, 'You feel your body change.')
      return {}
    end,
    deactivate = function(who, def)
      game.logPlayer(who, 'You are no longer transformed.')
      return true
    end,
    apply = function(who, t)
      local ObjectData = require 'mod.class.interface.ObjectData'
      local mf = ObjectData.mimicry_forms[t.mimic_form]
      local lev = who:getSkillLevel('MIMICRY')
      mf.apply(who, lev)
    end,
  }
end

define_cloak_talent('Mouse')
define_cloak_talent('Eagle')
define_cloak_talent('Wolf')
define_cloak_talent('Spider')
define_cloak_talent('Elder Ent')
define_cloak_talent('Vapour')
define_cloak_talent('Serpent')
define_cloak_talent('Mumak')
define_cloak_talent('Abomination')
]]

newTalent {
  name = 'Invisibility',
  short_name = 'CLOAK_MOUSE_INVIS',
  display_entity = Entity.new { image='talents/invisibility.png' },
  type = 'player/mimicry-cloak',
  mode = 'activated',
  is_spell = true,
  power = { level=30, stat='DEX', diff=20 },
  mana = 10,
  points = 0,
  action = function(self, t)
    local dur = 10 + (self.level or 0) + rng.range(1, 20) + (self.spellpower or 0)
    self:setEffect(self.EFF_INVISIBLE, dur, { power=50, merge='combine' })
    return true
  end,
  info = function(self, t)
    local base_dur = 10 + (self.level or 0) + (self.spellpower or 0)
    local ret = [[Use your exceptional hiding abilities as a mouse to beome effectively invisible (power 50) for %d+1d20 turns.  Duration increase with player level.]]
    return ret:format(base_dur)
  end,
}

newTalent {
  name = 'Spin Web',
  short_name = 'CLOAK_SPIDER_WEB',
  display_entity = Entity.new { image='talents/spin_web.png' },
  type = 'player/mimicry-cloak',
  mode = 'activated',
  is_spell = true,
  power = { level=25, stat='DEX', diff=20 },
  mana = 30,
  points = 0,
  action = function(self, t)
    spell.grow_terrain(self, 'WEB', 1 + math.floor((who.level or 0)/2))
    return true
  end,
  info = function(self, t)
    local r = 1 + math.floor((self.level or 0)/2)
    local ret = [[Spins webs in a radius of %d around you.  Radius increases with player level.]]
    return ret:format(r)
  end,
}
