-- $Id: zone.lua 3850 2020-04-05 20:13:34Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for Angband.

return {
  name = 'Angband',
  display_abbr = 'Ang',
  level_scheme = 'fixed',
  max_level = 61,
  level_range = { 67, 127 },
  width = 198,
  height = 66,
  principal = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { 'FLOOR' },
      fill = { 'WALL' },
      outer_wall = 'WALL',
      inner_wall = 'WALL',
      fill_step = 0,
      up = 'UP',
      down = 'DOWN',
      flags = {
	CAVERN = true,
	NO_EASY_MOVE = true,
	NO_RECALL = true,
	ADJUST_LEVEL_1_2 = true,
	ADJUST_LEVEL_1 = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=100, mode='any' },
      },
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  actor_adjust_level = function(zone, level, e)
    local lev = zone:level_adjust_level(level, 'actor')
    return rng.range(math.floor(lev/2), lev)
  end,
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
    [4] = {
      no_level_connectivity = true,
      level_name = [[Crypt]],
      enter_desc = [[Looks like a forgotten crypt...]],
      generator = {
	map = {
	  class='mod.class.generator.map.Static',
	  map = 'zones/crypt',
	  one_time_level = 'crypt',
	  flags = { DESC=true, NO_GENO=true, NO_NEW_MONSTER=true, SPECIAL=true, NO_STAIR=true, ASK_LEAVE=true, NO_TELEPORT=true },
	},
	actor = {
	  init_npc = table.NIL_MERGE,
	  respawn_in = table.NIL_MERGE,
	},
      },
    },
    [19] = {
      no_level_connectivity = true,
      level_name = [[Dim Gates]],
      enter_desc = [[Visions of death fill your mind.]],
      generator = {
	map = {
	  class='mod.class.generator.map.Static',
	  map = 'zones/dim-gates',
	  one_time_level = 'dim-gates',
	  flags = { DESC=true, NO_GENO=true, NO_NEW_MONSTER=true, SPECIAL=true, NO_STAIR=true, ASK_LEAVE=true, NO_TELEPORT=true },
	},
	actor = {
	  init_npc = table.NIL_MERGE,
	  respawn_in = table.NIL_MERGE,
	},
      },
    },
    [29] = {
      no_level_connectivity = true,
      level_name = [[Nameless]],
      enter_desc = [[You sense a powerful artifact here.]],
      generator = {
	map = {
	  class='mod.class.generator.map.Static',
	  map = 'zones/nameless',
	  one_time_level = 'nameless',
	  flags = { DESC=true, NO_GENO=true, NO_NEW_MONSTER=true, SPECIAL=true, NO_STAIR=true, ASK_LEAVE=true, NO_TELEPORT=true },
	},
	actor = {
	  init_npc = table.NIL_MERGE,
	  respawn_in = table.NIL_MERGE,
	},
      },
    },
    [33] = {
      generator = {
	map = {
	  down = 'DOWN_QUEST_SAURON',
	},
	actor = {
	  guardian = 'SAURON_THE_SORCERER',
	  guardian_level = 33,
	  guardian_precondition = function()
	    if game.player:isQuestStatus('one-ring', engine.Quest.PENDING) then return true end
	    if game.player:isQuestStatus('sauron', engine.Quest.PENDING) then return true end
	    return false
	  end,
	},
      },
    },
    [34] = {
      generator = {
	map = {
	  down = 'DOWN_QUEST_MORGOTH',
	},
	actor = {
	  guardian = 'MORGOTH_LORD_OF_DARKNESS',
	  guardian_level = 34,
	  guardian_precondition = function()
	    if game.player:isQuestStatus('morgoth', engine.Quest.PENDING) then return true end
	    return false
	  end,
	},
      },
    },
  },
}
