-- $Id: zone.lua 4091 2020-09-23 02:04:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

return {
  name = "Echoriath",
  level_scheme = 'fixed',
  max_level = 1,
  level_range = { 30, 30 },
  width = 50,
  height = 30,
  no_level_connectivity = true,
  no_recall_depth = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.EolQuestLevel',
      floor = { 'FLOOR' },
      fill = { 'WALL' },
      outer_wall = { 'WALL' },
      inner_wall = { 'WALL' },
      fill_step = 0,
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      -- Eol is placed by post_process() below.
      no_prepop = true,
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
      no_prepop = true,	-- This is just for random objects on the map
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
      n_traps = { 0, 0 },	-- we'll place traps in post_process() below
    }
  },

  post_process = function(level, zone)
    -- cf. quest_eol_gen_hook() in src/q_eol.c
    local plyr_spot = level:pickSpot{ type='quest', subtype='player_start' }
    local eol_spot = level:pickSpot{ type='quest', subtype='eol_start' }

    -- Place traps.
    local Map = require 'mod.class.Map'
    local map = level.map
    local t_gen = zone:getGenerator('trap', level)
    for x = map.w - 1, 2, -1 do
      for y = map.h - 1, 2, -1 do
	if map:checkEntity(x, y, Map.TERRAIN, 'clean_bold') and
	   (x ~= plyr_spot.x or y ~= plyr_spot.y) and
	   (x ~= eol_spot.x or y ~= eol_spot.y) and
	   rng.chance(18)
	then
	  t_gen:generateOne{ x=x, y=y }
	end
      end
    end
  end,
}
