-- $Id: zone.lua 4091 2020-09-23 02:04:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

return {
  name = 'Bree',
  level_scheme = 'fixed',
  max_level = 1,
  width = 198,
  height = 66,
  no_level_connectivity = true,
  all_remembered = true,
  all_lited = true,
  day_night = true,
  town = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.PlasmaWilderness',
      map = game.state:varTownDestroyed('bree', 'map'),
    },
    actor = game.state:varTownDestroyed('bree', 'actor_gen', {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 10,
      filters = {
	{ prob=100, mode='any' },
      },
    }),
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
      no_prepop = true,	-- No random objects in town.
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
      -- No random traps in town.
      n_traps = { 0, 0 },
    },
  },

  entry_points_from_zone = {
    ['barrow-downs'] = { type='zone-change', subtype='barrow-downs' },
    ['quest-bree-thieves'] = { type='quest', subtype='house' },
    ['quest-bree-wights'] = { type='quest', subtype='wights' },
    ['quest-bree-trolls'] = { type='quest', subtype='trolls' },
  },
  entry_point = util.town_wild_entry_point,

  quest_entrances = {
    ['bree-house'] = {
      type='quest', subtype='house', replace={
	[Quest.PENDING] = 'ENTER_BREE_HOUSE_QUEST',
	[Quest.COMPLETED] = 'ENTER_BREE_HOUSE_QUEST',
	-- Default grid is PERMA_WALL, which is what we wanted anyway.
	[Quest.DONE] = { trap='BREE_HOME' },
      },
    },
    ['bree-trolls'] = {
      type='quest', subtype='trolls', replace={
	[Quest.PENDING] = 'ENTER_BREE_TROLLS_QUEST',
	[Quest.COMPLETED] = 'ENTER_BREE_TROLLS_QUEST',
      },
    },
    ['bree-wights'] = {
      type='quest', subtype='wights', replace={
	[Quest.PENDING] = 'ENTER_BREE_WIGHTS_QUEST',
	[Quest.COMPLETED] = 'ENTER_BREE_WIGHTS_QUEST',
      },
    }
  },
  on_enter = function(to_zone, to_lev, from_lev, from_zone, params)
    util.juggle_quest_entrances(to_zone, to_lev, from_lev, from_zone, params)
    local Quest = require 'engine.Quest'
    local q = game.player:hasQuest('bree-nazgul')
    if q and q:isStatus(Quest.PENDING) then q:place_uvatha() end
  end,

  on_leave = util.on_leave_town_wild,
}
