-- $Id: zone.lua 3850 2020-04-05 20:13:34Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for The Void.

return {
  name = 'The Void',
  display_abbr = 'Vod',
  level_scheme = 'fixed',
  max_level = 23,
  level_range = { 128, 150 },
  width = 198,
  height = 66,
  effects = { 'EFF_ZONE_THE_VOID', 'EFF_AIRLESS', 'EFF_FORGET_MAP' },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { VOID=97, NETHER_MIST=3 } },
      fill = { weighted = { VOID={90,40}, NETHER_MIST={10,60} } },
      outer_wall = 'NETHER_MIST',
      inner_wall = 'NETHER_MIST',
      fill_step = 2,
      down = 'DOWN',
      flags = {
	EMPTY = true,
	FORGET = true,
	NO_BREATH = true,
	NO_EASY_MOVE = true,
	NO_RECALL_OUT = true,
	NO_RECALL = true,
	ADJUST_LEVEL_1_2 = true,
	ADJUST_LEVEL_1 = true,
	NO_STREAMERS = true,
	NO_SHAFT = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 20,
      respawn_in = 160,
      filters = {
	{ prob=1, mode='any' },
	{ prob=99, mode='or', flags={ 'UNDEAD', 'DEMON', 'DRAGON', 'NONLIVING', 'SPIRIT' } },
      },
      guardian = 'MELKOR_LORD_OF_DARKNESS',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=25, combat=25, magic=25, tools=25, junk=0 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  actor_adjust_level = function(zone, level, e)
    local lev = zone:level_adjust_level(level, 'actor')
    return rng.range(math.floor(lev/2), lev)
  end,
  alter_level_data = util.adjust_level,
  levels = {
    [21] = {
      generator = {
	map = {
	  down_branch = 'DOWN_TO_NETHER_REALM',
	},
      },
    },
    [23] = {
      generator = {
	map = {
	  flags = { NO_GENO=true },
	},
      },
    },
  },

  entry_point = function(self, lev, from_lev, from_zone)
    if not from_zone then
      return nil
    elseif from_zone.name == 'nether-realm' then
      -- Returning from a down branch; enter on a down-branch stair.
      return { type='branch', subtype='down' }
    elseif util.zone_is(from_zone, 'worldmap') or util.zone_is(from_zone, 'wilderness') then
      -- Coming from the world map (or Bree -> Barrow-Downs); start on an
      -- up stair.
      return { use='default_up' }
    else
      return nil
    end
  end,
}
