-- $Id: Background.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2015-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'

module(..., package.seeall, class.make)

-- Static
function _M:loadDefinition(file)
  local f, err = loadfile(file)
  if not f and err then error(err) end
  -- This isn't data we really need to keep around, so we fake up a table
  -- for newBackground() to accumulate definitions into and return that.
  local bgs = {}
  setfenv(f, setmetatable({
    newBackground = function(t) self:newBackground(t, bgs) end,
  }, {__index=_G}))
  f()
  return bgs
end

function _M:newBackground(t, bgs)
  assert(t.idx, 'no background idx')
  assert(t.text, 'no background text entries')
  for i, l in ipairs(t.text) do
    assert(l.prob, 'no background text prob in entry '..i)
    assert(l.prob > 0 and l.prob <= 100, 'out-of-range background text prob in entry '..i)
    assert(l.next, 'no background text next in entry '..i)
    assert(l.social, 'no background text social in entry '..i)
    assert(l.text, 'no background text text in entry '..i)
    if i > 1 then
      assert(l.prob > t.text[i-1].prob, 'misordered background text prob in entry '..i)
    end
  end
  assert(t.text[#t.text].prob == 100, 'incomplete background text entries')

  bgs[t.idx] = t
end

-- Static
function _M:generate(bgs, start)
  local social_class = rng.range(1, 4)
  local idx = start
  local text = ''

  while idx > 0 do
    local hit = false
    local p = rng.range(1, 100)
    for _, l in ipairs(bgs[idx].text) do
      if p <= l.prob then
	hit = true
	text = text .. l.text
	social_class = social_class + l.social
	idx = l.next
	break
      end
    end
    if not hit then
      error(('[BACKGROUND] no next idx from chart %d (p=%d)'):format(idx, p))
      break
    end
  end

  social_class = util.bound(social_class, 1, 100)
  return text, social_class
end
