-- $Id: HotkeysIcons.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2016-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- An attempt at making the engine's HotkeysIconsDisplay component more
-- modular and expandable (which we need, since we're adding thaumaturgy
-- hotkeys).  Our modified HotkeysIconsDisplay will do hotkey-type-specific
-- handling via us, using definitions loaded by :loadDefinition().

require 'engine.class'

module(..., package.seeall, class.make)

_M.hotkey_icons_def = {}

-- Static
function _M:loadDefinition(file)
  local f, err = loadfile(file)
  if not f and err then error(err) end
  setfenv(f, setmetatable({
    newHotkeyIcon = function(t) self:newHotkeyIcon(t) end,
  }, {__index=_G}))
  f()
end

-- Static
function _M:newHotkeyIcon(t)
  assert(t.type, 'no hotkey icon type')
  assert(type(t.display) == 'function', 'missing or malformed display() method')
  assert(type(t.dragSurface) == 'function', 'missing or malformed dragSurface() method')
  assert(type(t.tooltipText) == 'function', 'missing or malformed tooltipText() method')
  self.hotkey_icons_def[t.type] = t
end

-- Static
function _M:hasType(type)
  return self.hotkey_icons_def[type]
end

-- Static
function _M:display(ts, actor)
  if not ts[3] or not self.hotkey_icons_def[ts[3]] then return end
  return self.hotkey_icons_def[ts[3]].display(ts[1], actor)
end

-- Static
function _M:dragSurface(hk, actor)
  if not hk or not hk[1] or not self.hotkey_icons_def[hk[1]] then return nil end
  return self.hotkey_icons_def[hk[1]].dragSurface(hk[2], actor)
end

-- Static
function _M:tooltipText(hk, actor)
  if not hk or not hk[1] or not self.hotkey_icons_def[hk[1]] then return nil end
  return self.hotkey_icons_def[hk[1]].tooltipText(hk[2], actor)
end
