-- $Id: T2MazeLevel.lua 3665 2019-09-17 01:16:43Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2019 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Map = require 'mod.class.Map'
local T2LevelGenerator = require 'mod.class.generator.map.T2LevelGenerator'

module(..., package.seeall, class.inherit(T2LevelGenerator))

function _M:init(zone, map, level, data)
  T2LevelGenerator.init(self, zone, map, level, data)
end

local dxy = {
  [0] = {  1,  0 },
  [1] = {  0, -1 },
  [2] = { -1,  0 },
  [3] = {  0,  1 },
}

-- Loosely based on level_generate_maze() in src/gen_maze.c
function _M:layoutMap(lev, old_lev)
  self.cur_lev = lev
  self.dun_lev = self.zone:level_adjust_level(self.level, 'map')

  -- In the original algorithm, this is a 2-dimensional array of chars used
  -- as bitmasks for the four cardinal directions, with -1 used as a
  -- sentinel at the edges.  Since Lua doesn't have bit operators and
  -- building a 2D array would be fiddly, we "flatten" the array to 1D with
  -- index cleverness and use small 4-element lists at each location as
  -- fake bitmasks.
  self.scratch = {}
  local mw, mh = math.floor(self.map.w/2), math.floor(self.map.h/2)
  for y = 0, mh + 1 do
    self.scratch[self:idx(0, y)] = true
    self.scratch[self:idx(mw + 1, y)] = true
  end
  for x = 0, mw + 1 do
    self.scratch[self:idx(x, 0)] = true
    self.scratch[self:idx(x, mh + 1)] = true
  end

  local x, y, d = rng.range(1, mw), rng.range(1, mh), rng.range(0, 3)
  self:dig(x, y, d)

  -- The initial spot isn't necessarily properly populated; rebuild its
  -- direction map from scratch.
  local idx = self:idx(x, y)
  local cell = self.scratch[idx]
  for d = 0, 3 do
    local dcell = self.scratch[self:idx(x + dxy[d][1], y + dxy[d][2])]
    if dcell ~= true and dcell and dcell[(d + 2) % 4] then
      cell[d] = true
    end
  end
  self.scratch[idx] = cell

  -- Finally, place floors according to the data we've accumulated.
  for y = 1, mh do
    for x = 1, mw do
      cell = self.scratch[self:idx(x, y)]
      if cell then
	self.map(x*2, y*2, Map.TERRAIN, self:resolve('floor'))
	if cell[0] then
	  self.map(x*2 + 1, y*2, Map.TERRAIN, self:resolve('floor'))
	end
	if cell[3] then
	  self.map(x*2, y*2 + 1, Map.TERRAIN, self:resolve('floor'))
	end
      end
    end
  end

  -- Don't need this anymore.
  self.scratch = nil
end

function _M:idx(x, y)
  return x + y * self.map.w
end

-- Loosely based on dig() in src/gen_maze.c
function _M:dig(x, y, d)
  local idx = self:idx(x, y)
  -- First, open the wall of the new cell in the direction we came from.
  self.scratch[idx] = self.scratch[idx] or {}
  self.scratch[idx][(d + 2) % 4] = true

  -- Try to change direction (50% chance).
  if rng.chance(2) then d = rng.range(0, 3) end

  for _ = 1, 4 do
    local dx, dy = dxy[d][1], dxy[d][2]
    local didx = self:idx(x + dx, y + dy)
    if not self.scratch[didx] then
      -- Now, open the wall of the new cell in the direction we're going.
      self.scratch[idx][d] = true
      self:dig(x + dx, y + dy, d)
    end
    d = (d + 1) % 4
  end
end
