-- $Id: PlayerSanity.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'

module(..., package.seeall, class.make)

function _M:init(t)
  -- This will get fiddled by recalcSanity() below.
  self.max_sanity = t.max_sanity or 100
  self.sanity = t.sanity or self.max_sanity
  self.die_at_sanity = t.die_at_sanity or 0
end

function _M:healSanity(value, src)
  local was_hurt = self.sanity < self.max_sanity
  if self.onHealSanity then value = self:onHealSanity(value, src) end
  self.sanity = util.bound(self.sanity + value, self.die_at_sanity, self.max_sanity)
  self.changed = true
  return was_hurt
end

function _M:takeSanityHit(value, src, death_note)
  if self.onTakeSanityHit then value = self:onTakeSanityHit(value, src) end
  self.sanity = self.sanity - value
  self.changed = true
  if self.sanity <= self.die_at_sanity and not self.dead then
    if src.on_kill and src:on_kill(self) then return false, value end
    game.logSeen(self, '#{bold}#%s killed %s!{normal#', src.name:capitalize(), self.name)
    return self:die(src, death_note), value
  end
  return false, value
end

-- cf. calc_sanity() in src/xtra1.c
function _M:recalcSanity()
  local con_adj = self:getStatRelated('con_maxhp')
  local new_max = 5*(self.level + 1) + math.floor(con_adj/2)
  new_max = math.max(new_max, self.level + 1)

  if new_max ~= self.max_sanity then
    -- The original code doesn't check whether this drops sanity below
    -- zero, so neither do we.
    self.sanity = self.sanity + (new_max - self.max_sanity)
    self.max_sanity = new_max
    self.sanity = math.min(self.sanity, self.max_sanity)
  end
end
