-- $Id: BeastmasterResearch.lua 3064 2017-09-15 01:46:04Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local ListColumns = require 'engine.ui.ListColumns'
local ActorLearn = require 'mod.class.interface.ActorLearn'

module(..., package.seeall, class.inherit(Dialog))

function _M:init(actor, cost)
  self.actor = actor
  self.cost = cost

  Dialog.init(self, 'Select creature to research', game.w * 0.7, game.h * 0.7)
  local side_w = math.floor((self.iw/2) - 10)

  self.c_grps = ListColumns.new {
    width = math.floor((self.iw/2) - 10),
    height = self.ih,
    scrollbar = true,
    columns = {
      { name='Sym', width={30,'fixed'}, display_prop='sym' },
      { name='Category', width=100, display_name='name' },
    },
    -- From ident_info[] in src/cmd3.c, with some modifications.
    -- TODO Hardcoding this is Bad(TM), but not obvious how else to do it
    list = {
      { sym=[[A]], name=[[Angels]] },
      { sym=[[B]], name=[[Birds]] },
      { sym=[[C]], name=[[Canines]] },
      { sym=[[D]], name=[[Ancient Dragons/Wyrms]] },
      { sym=[[E]], name=[[Elementals]] },
      { sym=[[F]], name=[[Dragon Flies]] },
      { sym=[[G]], name=[[Ghosts]] },
      { sym=[[H]], name=[[Hybrids]] },
      { sym=[[I]], name=[[Insects]] },
      { sym=[[J]], name=[[Snakes]] },
      { sym=[[K]], name=[[Killer Beetles]] },
      { sym=[[L]], name=[[Liches]] },
      { sym=[[M]], name=[[Multi-Headed Reptiles]] },
      { sym=[[O]], name=[[Ogres]] },
      { sym=[[P]], name=[[Giant Humanoids]] },
      { sym=[[Q]], name=[[Quylthulgs (Pulsing Flesh Mounds)]] },
      { sym=[[R]], name=[[Reptiles/Amphibians]] },
      { sym=[[S]], name=[[Spiders/Scorpions/Ticks]] },
      { sym=[[T]], name=[[Trolls]] },
      { sym=[[U]], name=[[Major Demons]] },
      { sym=[[V]], name=[[Vampires]] },
      { sym=[[W]], name=[[Wights/Wraiths/etc]] },
      { sym=[[X]], name=[[Xorns/Xarens/etc]] },
      { sym=[[Y]], name=[[Yeti]] },
      { sym=[[Z]], name=[[Zephyr Hounds]] },
      { sym=[[a]], name=[[Ants]] },
      { sym=[[b]], name=[[Bats]] },
      { sym=[[c]], name=[[Centipedes]] },
      { sym=[[d]], name=[[Dragons]] },
      { sym=[[e]], name=[[Floating Eyes]] },
      { sym=[[f]], name=[[Felines]] },
      { sym=[[g]], name=[[Golems]] },
      { sym=[[h]], name=[[Hobbits/Elves/Dwarves]] },
      { sym=[[i]], name=[[Icky Things]] },
      { sym=[[j]], name=[[Jellies]] },
      { sym=[[k]], name=[[Kobolds]] },
      { sym=[[l]], name=[[Lice]] },
      { sym=[[m]], name=[[Molds]] },
      { sym=[[n]], name=[[Nagas]] },
      { sym=[[o]], name=[[Orcs]] },
      { sym=[[p]], name=[[People/Humans]] },
      { sym=[[q]], name=[[Quadrupeds]] },
      { sym=[[r]], name=[[Rodents]] },
      { sym=[[s]], name=[[Skeletons]] },
      { sym=[[t]], name=[[Townspeople]] },
      { sym=[[u]], name=[[Minor Demons]] },
      { sym=[[v]], name=[[Vortices]] },
      { sym=[[w]], name=[[Worms/Worm-Masses]] },
      { sym=[[y]], name=[[Yeeks]] },
      { sym=[[z]], name=[[Zombies/Mummies]] },
      { sym=[[|]], name=[[Animated weapons]] },
      { sym=[[~]], name=[[Aquatic creatures]] },
      { sym=[[#]], name=[[Trees/plants/walls]] },
      { sym=[[$]], name=[[Creeping coins]] },
      { sym=[[(]], name=[[Animated armor]] },
      { sym=[[*]], name=[[Magic orbs]] },
      { sym=[[+]], name=[[Door mimics]] },
      { sym=[[,]], name=[[Mushroom patches]] },
      { sym=[[.]], name=[[Floor/ceiling creatures or spirits]] },
    },
    fct = function(item, sel) self:setCategory(item) end
  }

  self.c_mons = ListColumns.new {
    width = math.floor((self.iw/2) - 10),
    height = self.ih,
    scrollbar = true,
    columns = {
      { name='Creature', width=100, display_name='name' },
    }
    list = {}, -- Will be populated by :setCategory()
    fct = function(item, sel) self:learnMon(item) end,
  }

  local hs = Separator.new { dir='horizontal', size=self.ih }
  self:loadUI {
    { left=0, top=0, ui=self.c_grps },
    { left=self.c_grps, top=0, ui=hs },
    { right=0, top=0, ui=self.c_mons },
  }
  self:setFocus(self.c_grps)
  self:setupUI()

  self.key:addBind('EXIT', function() game:unregisterDialog(self) end)
end

function _M:setCategory(item)
  if not item or not item.sym then return end
  local list = {}
  for _, e in ipairs(game.zone.npc_list) do
    if e.display == item.sym then
      list[#list+1] = { name=e.name, tag=e.define_as }
    end
  end
  self.c_mons:setList(list)
  self:setFocus(self.c_mons)
end

function _M:learnMon(item)
  if item and item.tag then
    ActorLearn:learnAllFlagsFor(item.tag)
    game:unregisterDialog(self)
    Dialog:simpleLongPopup(item.name, game.zone.npc_list[e.tag]:getDesc(), 600, nil, false, 0.8)
  end
end
