-- $Id: ConfigureItemDialog.lua 3602 2019-05-11 19:24:29Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2019 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require "engine.class"
local Dialog = require "engine.ui.Dialog"
local Textzone = require "engine.ui.Textzone"
local Button = require "engine.ui.Button"
local Separator = require "engine.ui.Separator"

module(..., package.seeall, class.inherit(Dialog))

function _M:init(object)
  self.object = object

  Dialog.init(self, 'Configure Item', 1, 1)

  self.c_name = Textzone.new {
    width = 500,
    auto_height = true,
    text = object:getName(),
  }
  self.c_ok = Button.new {
    text = 'OK',
    fct = function() self:apply() end,
  }
  self.c_cancel = Button.new {
    text = 'Cancel',
    fct = function() game:unregisterDialog(self) end,
  }
  local sep = Separator.new { dir='vertical', size=500 }

  local layout
  self.config_cbs, layout = ui_util.config_checkboxes(sep, self.object)
  table.insert(layout, 1, { left=0, top=0, ui=self.c_name })
  table.insert(layout, 2, { hcenter=0, top=self.c_name, ui=sep })
  layout[#layout+1] = { left=3, bottom=3, ui=self.c_ok }
  layout[#layout+1] = { right=3, bottom=3, ui=self.c_cancel }

  self:loadUI(layout)
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end
  }
end

function _M:apply()
  local ObjectData = require 'mod.class.interface.ObjectData'
  game:unregisterDialog(self)
  self.object:forAllStack(function(o)
    for tag, cb in pairs(self.config_cbs) do
      local def = ObjectData.config_types[tag]
      local ok = not def.applicable or def.applicable(o)
      o:setConfig(tag, ok and cb.checked)
    end
  end)
end
