-- $Id: LibraryQuestSpellbook.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local Button = require 'engine.ui.Button'
local ListColumns = require 'engine.ui.ListColumns'
local Textzone = require 'engine.ui.Textzone'
local TextzoneList = require 'engine.ui.TextzoneList'
local Quest = require 'engine.Quest'

module(..., package.seeall, class.inherit(Dialog))

-- cf. library_quest.bookable_spells{} in lib/scpt/library.lua
_M.spells= {
  -- Mana
  'MANATHRUST',
  'REMOVE_CURSE',
  -- Fire
  'GLOBE_LIGHT',
  'FIRE_GOLEM',
  'FIREFLASH',
  'FIREWALL',
  -- Water
  'GEYSER',
  'VAPOR',
  'ENT_POTION',
  -- Air
  'NOXIOUS_CLOUD',
  'POISON_BLOOD',
  -- Earth,
  'STONE_SKIN',
  'DIG',
  -- Meta
  'RECHARGE',
  'DISPERSE_MAGIC',
  -- Conveyance
  'PHASE_DOOR',
  'DISARM',
  'TELEPORT',
  -- Divination
  'SENSE_MONSTERS',
  'SENSE_HIDDEN',
  'IDENTIFY',
  'REVEAL_WAYS',
  'VISION',
  -- Temporal
  'MAGELOCK',
  'SLOW_MONSTER',
  'ESS_SPEED',
  -- Mind
  'CHARM',
  'CONFUSE',
  'ARMOR_FEAR',
  'STUN',
  -- Nature
  'GROW_TREES',
  'HEALING',
  'RECOVERY',
  -- Eru
  'ERU_SEE_MUSIC',
  'ERU_LISTEN_MUSIC',
  -- Manwe
  'MANWE_BLESSING',
  'MANWE_WIND_SHIELD',
  -- Yavanna
  'YAVANNA_CHARM_ANIMAL',
  'YAVANNA_GROW_GRASS',
  'YAVANNA_TREE_ROOTS',
  -- Tulkas
  'TULKAS_DIVINE_AIM',
  'TULKAS_WHIRLWIND',
  -- Melkor
  'MELKOR_CURSE',
  'MELKOR_CORPSE_EXPL',
  -- Udun
  'DRAIN',
}

function _M:init(actor)
  self.actor = actor
  self.sel_spells = {}

  Dialog.init(self, 'Quest:  Library quest', 1, 1)

  self:generateList()
  self.c_spells = ListColumns.new {
    width = self.max_w + 10,
    nb_items = 20,
    scrollbar = true,
    columns = {
      { name='Spell', width=100, display_prop='name' },
    },
    list = self.list,
    select = function(item, sel) self:selected(item) end,
    fct = function(item, sel) self:use(item) end,
  }

  self.c_sel_spells = TextzoneList.new {
    width = 300,
    height = self.font_h*6,
    no_color_bleed = true,
  }
  self.c_tal_desc = TextzoneList.new {
    width = 300,
    height = self.c_spells.h - self.c_sel_spells.h,
    no_color_bleed = true,
  }
  self:selected(self.list[self.c_spells.sel])

  self.c_desc = Textzone.new {
    width = self.c_spells.w + self.c_sel_spells.w + 10,
    auto_height = true,
    no_color_bleed = true,
    text = [[Select a spell to add or remove it from your spellbook.  You may select up to three spells.]],
  }

  self.c_ok = Button.new {
    text = 'Done',
    fct = function() self:finish() end,
  }

  self:loadUI {
    { hcenter=0, top=0, ui=self.c_desc },
    { left=0, top=self.c_desc, ui=self.c_spells },
    { left=self.c_spells, top=self.c_desc, ui=self.c_sel_spells },
    { left=self.c_spells, top=self.c_sel_spells, ui=self.c_tal_desc },
    { hcenter=0, right=0, bottom=0, ui=self.c_ok, hidden=true },
  }
  self:setFocus(self.c_spells)
  self:setupUI(true, true)
end

function _M:selected(item)
  if not item.t then return end
  local text = tstring {
    {'color','GOLD'}, {'font','bold'}, item.t.name,
    {'font','normal'}, {'color','LAST'}, true
  }
  text:merge(self.actor:getTalentFullDescription(item.t))
  -- This gets called during :init() before we're fully assembled, so we
  -- have to check.
  if self.c_tal_desc then
    self.c_tal_desc:switchItem(item.t.id, tostring(text))
  end
end

function _M:use(item)
  if not item.t then return end
  self.sel_spells[item.t.id] = not self.sel_spells[item.t.id]
  self:updateSelected()
end

function _M:spellList(field)
  local sel = {}
  for _, item in ipairs(self.list) do
    if self.sel_spells[item.t.id] then sel[#sel+1] = item.t[field] end
  end
  return sel
end

function _M:updateSelected()
  local sel = self:spellList('name')
  local text = 'No selected spells'
  if #sel > 0 then
    text = 'Selected spells:\n· ' .. table.concat(sel, '\n· ')
  end
  self.c_sel_spells:switchItem('dummy', text, true)
  self:generateList()
  self.c_spells:setList(self.list, true)
  self:toggleDisplay(self.c_ok, #sel > 0)
end

function _M:finish()
  local sel = self:spellList('name')
  local sel_txt = table.concat(sel, '\n· ')
  local txt = ('You have selected %d spell%s:\n· %s\nCreate spellbook?'):format(#sel, (#sel == 1 and '' or 's'), sel_txt)
  local function cb(ok)
    if ok then
      game:unregisterDialog(self)
      local o = game.zone:makeEntityByName(game.level, 'object', 'TOME_OF')
      if o then
	o.carried_talents = self:spellList('id')
	o:identify(true, self.actor)
	o:setFoundHere {
	  type = 'custom',
	  custom = 'You received it as a reward for helping the Minas Anor librarians retake their deep stacks.',
	}
	game.logPlayer(self.actor, 'The librarian gives you:  %s', o:getName())
	self.actor:addOrDropObject(self.actor.INVEN_INVEN, o, nil, nil, {vocal_drop=true})
	self.actor:sortInven()
	self.actor:setQuestStatus('minas-anor-library', Quest.DONE)
      end
    end
  end
  Dialog:yesnoLongPopup('Confirm spells', txt, 400, cb)
end

function _M:generateList()
  local list = {}
  self.max_w = 0

  for _, tag in ipairs(self.spells) do
    local t_id = self.actor['T_'..tag]
    local t = self.actor:getTalentFromId(t_id)
    local name = t.name
    if self.sel_spells[t_id] then name = '#LIGHT_GREEN#'..name..'#LAST#' end
    list[#list+1] = { name=name, t=t }
    local w, _ = self.font:size(t.name)
    self.max_w = math.max(self.max_w, w)
  end

  self.list = list
end
