-- $Id: QuickAutomatizerRule.lua 3774 2019-12-29 20:13:34Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local Dropdown = require 'engine.ui.Dropdown'
local Textzone = require 'engine.ui.Textzone'
local Checkbox = require 'engine.ui.Checkbox'
local RadioButton = require 'mod.dialogs.ui.RadioButton'
local Button = require 'engine.ui.Button'
local Automatizer = require 'mod.class.Automatizer'

module(..., package.seeall, class.inherit(Dialog))

function _M:init(o)
  self.object = o

  self.c_action = Dropdown.new {
    width = 100,
    list = {
      { name='Destroy', action='destroy' },
      { name='Pick up', action='pickup' },
    },
    nb_items = 2,
    fct = function(item) end,
  }
  local c_label = Textzone.new {
    text = ' objects with:',
    auto_height = true,
    auto_width = true,
  }

  local name = o:getName{no_count=true, force_qty=1, no_mods=true, no_pval=true, no_inscr=true}
  self.c_rules = {
    RadioButton.new {
      title = ([[Type "%s"]]):format(tostring(o.type)),
      radio_group = 'ruie_types',
      default = false,
    },
    RadioButton.new {
      title = ([[Type/subtype "%s/%s"]]):format(tostring(o.type), tostring(o.subtype)),
      radio_group = 'ruie_types',
      default = false,
    },
    RadioButton.new {
      title = ([[Base type "%s"]]):format(tostring(o.define_as)),
      radio_group = 'ruie_types',
      default = true,
    },
    RadioButton.new {
      title = ([[Name "%s"]]):format(name),
      radio_group = 'ruie_types',
      default = false,
    },
  }

  local status = Automatizer.test_values.status(o)
  self.c_status = Checkbox.new {
    title = ([[and status "%s"]]):format(status),
    default = false,
    on_change = function(s) end,
  }
  self.c_global = Checkbox.new {
    title = [[Apply to all characters]],
    default = false,
    on_change = function(s) end,
  }

  self.c_ok = Button.new {
    text = 'OK',
    fct = function() self:createRule() end
  }
  self.c_cancel = Button.new {
    text = 'Cancel',
    fct = function() self.key:triggerVirtual('EXIT') end
  }

  Dialog.init(self, 'Quick Automatizer Rule', 1, 1)

  local uis = {
    { left=0, top=0, ui=self.c_action },
    { left=self.c_action, top=0, ui=c_label },
  }
  local prev_ui = self.c_action
  for _, ui in ipairs(self.c_rules) do
    uis[#uis+1] = { left=0, top=prev_ui, ui=ui }
    prev_ui = ui
  end
  if status and status ~= '' then
    uis[#uis+1] = { left=0, top=prev_ui, ui=self.c_status }
    prev_ui = self.c_status
  end
  uis[#uis+1] = { left=0, top=prev_ui, ui=self.c_global }
  uis[#uis+1] = { left=0, bottom=0, ui=self.c_ok }
  uis[#uis+1] = { left=self.c_ok, bottom=0, ui=self.c_cancel }

  self:loadUI(uis)
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end,
  }
end

function _M:createRule()
  game:unregisterDialog(self)

  local name, match
  local o_type = tostring(self.object.type)
  local o_subtype = tostring(self.object.subtype)
  local o_tag = tostring(self.object.define_as)
  local o_name = self.object:getName{no_count=true, force_qty=1, no_mods=true, no_pval=true, no_inscr=true}
  local o_status = Automatizer.test_values.status(self.object)

  if self.c_rules[1].checked then
    name = ([[Auto type '%s']]):format(o_type)
    match = ([[type == '%s']]):format(o_type)
  elseif self.c_rules[2].checked then
    name = ([[Auto type/subtype '%s/%s']]):format(o_type, o_subtype)
    match = ([[full_type == '%s/%s']]):format(o_type, o_subtype)
  elseif self.c_rules[3].checked then
    name = ([[Auto base type '%s']]):format(o_tag)
    match = ([[define_as == '%s']]):format(o_tag)
  elseif self.c_rules[4].checked then
    name = ([[Auto name '%s']]):format(o_name)
    match = ([[name == %q]]):format(o_name)
  else
    return
  end
  if self.c_status.checked then
    name = name .. ' + ' .. o_status
    match = match .. ([[ and status == '%s']]):format(o_status)
  end

  Automatizer:newAutomatizerRule {
    name = Automatizer:decollideRuleName(name),
    action = self.c_action.c_list.list[self.c_action.c_list.sel].action,
    match = match,
    auto = true,
    global = self.c_global.checked,
  }
  Automatizer:writeAutoRules()
end
