-- $Id: Quit.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local List = require 'engine.ui.List'
local Savefile = require 'engine.Savefile'

module(..., package.seeall, class.inherit(engine.ui.Dialog))

function _M:init()
  local w, h = 250, 150
  Dialog.init(self, 'Save Character', 250, 150)
  self.c_list = List.new {
    width = w,
    height = h,
    list = {
      {
	name = 'Continue playing',
	fct = function(_)
	  game:unregisterDialog(self)
	  game.quit_dialog = nil
	end
      },
      {
	name = 'Save and continue playing',
	fct = function(_)
	  game:unregisterDialog(self)
	  game.quit_dialog = nil
	  savefile_pipe:push(game.save_name, "game", game)
	end
      },
      {
	name = 'Save and return to main menu',
	fct = function(_)
	  game:unregisterDialog(self)
	  savefile_pipe:push(game.save_name, "game", game)
	  util.showMainMenu()
	end
      },
      {
	name = 'Save and exit game',
	fct = function(_)
	  game:unregisterDialog(self)
	  savefile_pipe:push(game.save_name, "game", game)
	  savefile_pipe:forceWait()
	  engine.GameTurnBased.onExit(game)
	end
      },
      {
	name = 'Abandon character and return to main menu',
	fct = function(_)
	  game:unregisterDialog(self)
	  game.quit_dialog = nil
	  self:confirmQuit(false)
	end
      },
      {
	name = 'Abandon character and exit game',
	fct = function(_)
	  game:unregisterDialog(self)
	  game.quit_dialog = nil
	  self:confirmQuit(true)
	end
      },
    },
    fct = function() self.key:triggerVirtual('ACCEPT') end,
  }
  self:loadUI {
    { left = 3, top = 3, ui = self.c_list }
  }
  self.key:addBind('EXIT', function()
    game:unregisterDialog(self)
    game.quit_dialog = nil
  end)
  self.key:addBind('ACCEPT', function()
    local item = self.c_list.list[self.c_list.sel]
    if item and item.fct then item.fct(item) end
  end)
  self:setFocus(self.c_list)
  self:setupUI(true, true)
end

function _M:confirmQuit(full_exit)
  local text = "#{bold}#WARNING:#{normal}#  This will remove the savefile for this character, and you will be unable to play it further.\n\nProceed anyway?"
  local callback = function(ret)
    if not ret then
      local save = Savefile.new(game.save_name)
      save:delete()
      save:close()
      if full_exit then
	engine.GameTurnBased.onExit(game)
      else
	util.showMainMenu()
      end
    end
  end
  Dialog:yesnoLongPopup('Quit Permanently?', text, 400, callback, "Don't quit", 'Quit')
end
