-- $Id: ShowLore.lua 3747 2019-10-30 02:27:29Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2019 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local ListColumns = require 'engine.ui.ListColumns'
local TextzoneList = require 'engine.ui.TextzoneList'
local Separator = require 'engine.ui.Separator'

module(..., package.seeall, class.inherit(Dialog))

function _M:init(title, actor)
  self.actor = actor
  local n_known = table.count(actor.lore_known)
  local total = #actor.lore_defs

  Dialog.init(self, ('%s (%d/%d)'):format(title or 'Lore', n_known, total), game.w * 0.8, game.h * 0.8)

  self:generateList()

  local vsep = Separator.new {
    dir = 'horizontal',
    size = self.ih - 10,
  }
  self.c_desc = TextzoneList.new {
    width = math.floor(self.iw/2 - vsep.w/2),
    height = self.ih,
    scrollbar=true,
  }
  self.c_list = ListColumns.new {
    width = math.floor(self.iw/2 - vsep.w/2),
    height = self.ih,
    scrollbar = true,
    sortable = true,
    columns = {
      { name='', width={40,'fixed'}, display_prop='order', sort='order' },
      { name='Lore', width=60, display_prop='name', sort='name' },
      { name='Category', width=40, display_prop='cat', sort='cat' },
    },
    list = self.list,
    fct = function(item) end,
    select = function(item, sel) self:select(item) end,
  }

  self:loadUI {
    { left=0, top=0, ui=self.c_list },
    { right=0, top=0, ui=self.c_desc },
    { hcenter=0, top=5, ui=vsep },
  }
  self:setFocus(self.c_list)
  self:setupUI()
  self:select(self.list[1])

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end,
  }
end

function _M:generateList()
  local list = {}

  for id, _ in pairs(self.actor.lore_known) do
    local l = self.actor:getLore(id)
    -- TODO Maybe filter on search?
    list[#list+1] = {
      name = l.name,
      desc = l.lore,
      cat = l.category,
      order = l.order,
      lore = l,
    }
  end
  table.sort(list, function(a, b) return a.order < b.order end)
  self.list = list
  if self.c_list then self.c_list:setList(list) end
end

function _M:select(item)
  if item then
    local text = ("#GOLD#Category:#AQUAMARINE# %s\n#GOLD#Found as:#0080FF# %s\n#GOLD#Text:#ANTIQUE_WHITE# %s"):format(item.cat, item.name, item.desc)
    self.c_desc:switchItem(item, text)
  end
end
