-- $Id: TalentChat.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- A modified version of engine.dialogs.Chat designed to be compatible with
-- ActorTalents:talentDialog() [q.v.].  In particular, the special :unload()
-- method installed on the dialog by :talentDialog() will be propagated
-- across all the individual stage dialogs.  The action() methods in your
-- chat definition should call :talentDialogReturn() at the appropriate
-- points.
require 'engine.class'
local ChatDialog = require 'engine.dialogs.Chat'

module(..., package.seeall, class.inherit(ChatDialog))

function _M:init(chat, id)
  ChatDialog.init(self, chat, id)

  -- Dodgy Hack(TM):  Find the ActorFrames in our UI and re-aim their
  -- 'tiles' field at the map's Tiles generator.
  if game.level and game.level.map.tiles then
    for _, ui in ipairs(self.uis) do
      if ui.ui and ui.ui.tiles then ui.ui.tiles = game.level.map.tiles end
    end
  end
end

-- Replace the parent's :regen() to copy across the :unload() method
-- installed by ActorTalents:talentDialog()
function _M:regen()
  local d = new(self.chat, self.cur_id)
  d.__showup = false
  d.unload = self.unload
  -- AFAICT Game:replaceDialog() doesn't call the outgoing dialog's
  -- :unload() method, but just in case...
  self.unload = nil
  game:replaceDialog(self, d)
  self.next_dialog = d
end
