-- $Id: GenerateItemsDialog.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local Button = require 'engine.ui.Button'
local Numberbox = require 'engine.ui.Numberbox'
local Checkbox = require 'engine.ui.Checkbox'
local Object = require 'mod.class.Object'

module(..., package.seeall, class.inherit(Dialog))

function _M:init()
  local cboxes = { 'c_good', 'c_great', 'c_randart' }
  local function radio(selected)
    for _, cbox in ipairs(cboxes) do
      self[cbox].checked = (cbox == selected)
    end
  end

  self.c_nitems = Numberbox.new {
    title = 'Items to generate: ',
    number = 100,
    min = 1,
    max = 1000,
    chars = 4,
    fct = function(text) end,
  }
  self.c_good = Checkbox.new {
    title = 'Force good',
    default = false,
    on_change = function(s) radio('c_good') end
  }
  self.c_great = Checkbox.new {
    title = 'Force great',
    default = false,
    on_change = function(s) radio('c_great') end
  }
  self.c_randart = Checkbox.new {
    title = 'Only randarts',
    default = false,
    on_change = function(s) radio('c_great') end
  }
  self.c_gen = Button.new {
    text = 'Generate',
    fct = function() self:generateItems() end,
  }
  self.c_exit = Button.new {
    text = 'Generate',
    fct = function() self.key:triggerVirtual('EXIT') end,
  }

  Dialog.init(self, 'Item Generation Test', 800, 600)

  self:loadUI {
    { left=0, top=0, ui=self.c_nitems },
    { left=0, top=self.c_nitems, ui=self.c_good },
    { left=0, top=self.c_good, ui=self.c_great },
    { left=0, top=self.c_great, ui=self.c_randart },
    { right=0, bottom=0, ui=self.c_exit },
    { right=self.c_exit, bottom=0, ui=self.c_gen },
  }
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end,
  }
end

function _M:on_register()
  game:onTickEnd(function() self.key:unicodeInput(true) end)
end

function _M:generateItems()
  if not self.c_nitems.number then return end

  local now = os.date("%Y%m%d-%H%M%S")

  fs.mkdir("/character-dumps")
  local file = "/character-dumps/itemgen-"..now..".txt"
  local fff = fs.open(file, "w")

  local objs = {}

  for _ in 1, self.c_nitems.number do
    local RF = Object.resolve_flags
    local o
    if self.c_randart.checked then
      o = obj_util:create_randart()
    else
      RF.force_good = self.c_good.checked
      RF.force_great = self.c_great.checked
      -- We count on our level's default object filter here.
      local o = self.zone:makeEntity(self.level, 'object', nil, nil, true)
    end
    if o then
      -- Test duplicate prevention.
      o:added()
      o:identify('full')
      objs[#objs+1] = o
  end

  fff:write(("%d of %d %s generated:\n\n"):format(#objs, self.c_nitems.number, (self.c_randart.checked and 'randarts' or 'objects')))
  for _, o in ipairs(objs) do
    fff.write(("  %s\n"):format(o:getName()))
  end

  -- Put all these back in circulation (unless you have preserve mode off).
  for _, o in ipairs(objs) do o:removed() end

  fff:close()
  Dialog:simplePopup("Items Generated", 'Test items listed in file '..fs.getRealPath(file))
end
